@@ -540,6 +540,17 @@ TEST (DecoderDeblocking, DeblockingInit) {
   DeblockingInit (&sDBFunc, 0x000004);
   DB_FUNC_CPUFLAG (AArch64_neon)
 #endif
+
+#ifdef HAVE_MMI
+  // pure C
+  DeblockingInit (&sDBFunc, 0x00000000);
+  DB_FUNC_CPUFLAG (c)
+
+  // mmi
+  DeblockingInit (&sDBFunc, 0x00000001);
+  DB_FUNC_CPUFLAG (mmi)
+#endif
+
 }
 
 TEST (DecoderDeblocking, WelsDeblockingFilterSlice) {
@@ -765,9 +776,11 @@ TEST (DecoderDeblocking, FilteringEdgeLumaHV) {
 
 /////////// Bs calculation functions
 TEST (DecoderDeblocking, DeblockingBsMarginalMBAvcbase) {
-  // uint32_t DeblockingBsMarginalMBAvcbase (PDqLayer pCurDqLayer, int32_t iEdge, int32_t iNeighMb, int32_t iMbXy)
+  // uint32_t DeblockingBsMarginalMBAvcbase (PDeblockingFilter pFilter, PDqLayer pCurDqLayer, int32_t iEdge, int32_t iNeighMb, int32_t iMbXy)
   /* Calculate the Bs equal to 2 or 1 */
   SDqLayer sDqLayer;
+  SDeblockingFilter sFilter;
+  int i;
 
   // Only define 2 MBs here
   int8_t iNoZeroCount[24 * 2]; // (*pNzc)[24]
@@ -776,6 +789,8 @@ TEST (DecoderDeblocking, DeblockingBsMarginalMBAvcbase) {
   uint32_t uiBSx4;
   uint8_t* pBS = (uint8_t*) (&uiBSx4);
 
+  PPicture iFilterPics[2][MAX_DPB_COUNT]; // Dummy reference pictures list
+
   sDqLayer.pNzc = (int8_t (*)[24])iNoZeroCount;
   sDqLayer.pRefIndex[0] = (int8_t (*)[16])&iLayerRefIndex[0];
   sDqLayer.pRefIndex[1] = (int8_t (*)[16])&iLayerRefIndex[1];
@@ -786,7 +801,14 @@ TEST (DecoderDeblocking, DeblockingBsMarginalMBAvcbase) {
   bool bTSize8x8Flag[50] = {false};
   sDqLayer.pTransformSize8x8Flag = bTSize8x8Flag;
   memset (bTSize8x8Flag, 0, sizeof (bool) * 50);
+  // Dummy picture list pointers to 1..MAX_DPB_COUNT
+  // the pointer values don't need to be valid, just different
+  for (i = 0; i < MAX_DPB_COUNT; i++)
+    iFilterPics[0][i] = iFilterPics[1][i] = (PPicture) (iFilterPics[0] + i);
 
+  sFilter.pRefPics[0] = iFilterPics[0];
+  sFilter.pRefPics[1] = iFilterPics[1];
+  sDqLayer.pDec = NULL;
 #define UT_DB_CLEAN_STATUS \
   memset(iNoZeroCount, 0, sizeof(int8_t)*24*2); \
   memset(iLayerRefIndex, 0, sizeof(int8_t)*2*16*2); \
@@ -807,79 +829,83 @@ TEST (DecoderDeblocking, DeblockingBsMarginalMBAvcbase) {
       // (1) iEdge == 0, current block NoZeroCount != 0
       UT_DB_CLEAN_STATUS
       iNoZeroCount[0 * 24 + iCurrBlock] = 1; // Current MB_block position
-      SET_REF_VALUE(2, iPos);
-      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sDqLayer, iEdge, 1,
+      SET_REF_VALUE (2, iPos);
+      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sFilter, &sDqLayer, iEdge, 1,
                    0) == uiBSx4) << iEdge << " " << iPos << " NoZeroCount!=0";
 
       // (2) iEdge == 0, neighbor block NoZeroCount != 0
       UT_DB_CLEAN_STATUS
       iNoZeroCount[1 * 24 + iNeighborBlock ] = 1; // Neighbor MB_block position
-      SET_REF_VALUE(2, iPos);
-      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sDqLayer, iEdge, 1,
+      SET_REF_VALUE (2, iPos);
+      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sFilter, &sDqLayer, iEdge, 1,
                    0) == uiBSx4) << iEdge << " " << iPos << " NoZeroCount!=0";
 
       // (3) iEdge == 0, reference idx diff
       UT_DB_CLEAN_STATUS
       iLayerRefIndex[0][0 * 16 + iCurrBlock] = 0;
       iLayerRefIndex[0][1 * 16 + iNeighborBlock] = 1;
-      SET_REF_VALUE(1, iPos);
-      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sDqLayer, iEdge, 1,
+      SET_REF_VALUE (1, iPos);
+      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sFilter, &sDqLayer, iEdge, 1,
                    0) == uiBSx4) << iEdge << " " << iPos << " Ref idx diff";
 
       // (4) iEdge == 0, abs(mv diff) < 4
       UT_DB_CLEAN_STATUS
       iLayerMv[0][0 * 16 + iCurrBlock][0] = rand() % 4;
-      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sDqLayer, iEdge, 1, 0) == 0) << iEdge << " " << iPos << " diff_mv < 4";
+      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sFilter, &sDqLayer, iEdge, 1,
+                   0) == 0) << iEdge << " " << iPos << " diff_mv < 4";
 
       UT_DB_CLEAN_STATUS
       iLayerMv[0][0 * 16 + iCurrBlock][1] = rand() % 4;
-      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sDqLayer, iEdge, 1, 0) == 0) << iEdge << " " << iPos << " diff_mv < 4";
+      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sFilter, &sDqLayer, iEdge, 1,
+                   0) == 0) << iEdge << " " << iPos << " diff_mv < 4";
 
       UT_DB_CLEAN_STATUS
       iLayerMv[0][1 * 16 + iNeighborBlock][0] = rand() % 4;
-      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sDqLayer, iEdge, 1, 0) == 0) << iEdge << " " << iPos << " diff_mv < 4";
+      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sFilter, &sDqLayer, iEdge, 1,
+                   0) == 0) << iEdge << " " << iPos << " diff_mv < 4";
 
       UT_DB_CLEAN_STATUS
       iLayerMv[0][1 * 16 + iNeighborBlock][1] = rand() % 4;
-      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sDqLayer, iEdge, 1, 0) == 0) << iEdge << " " << iPos << " diff_mv < 4";
+      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sFilter, &sDqLayer, iEdge, 1,
+                   0) == 0) << iEdge << " " << iPos << " diff_mv < 4";
 
       // (5) iEdge == 0, abs(mv diff) > 4
       UT_DB_CLEAN_STATUS
       iLayerMv[0][0 * 16 + iCurrBlock][0] = 4;
-      SET_REF_VALUE(1, iPos);
-      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sDqLayer, iEdge, 1,
+      SET_REF_VALUE (1, iPos);
+      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sFilter, &sDqLayer, iEdge, 1,
                    0) == uiBSx4) << iEdge << " " << iPos << " diff_mv == 4";
 
       UT_DB_CLEAN_STATUS
       iLayerMv[0][0 * 16 + iCurrBlock][1] = 4;
-      SET_REF_VALUE(1, iPos);
-      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sDqLayer, iEdge, 1,
+      SET_REF_VALUE (1, iPos);
+      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sFilter, &sDqLayer, iEdge, 1,
                    0) == uiBSx4) << iEdge << " " << iPos << " diff_mv == 4";
 
       UT_DB_CLEAN_STATUS
       iLayerMv[0][1 * 16 + iNeighborBlock][0] = 4;
-      SET_REF_VALUE(1, iPos);
-      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sDqLayer, iEdge, 1,
+      SET_REF_VALUE (1, iPos);
+      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sFilter, &sDqLayer, iEdge, 1,
                    0) == uiBSx4) << iEdge << " " << iPos << " diff_mv == 4";
 
       UT_DB_CLEAN_STATUS
       iLayerMv[0][1 * 16 + iNeighborBlock][1] = 4;
-      SET_REF_VALUE(1, iPos);
-      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sDqLayer, iEdge, 1,
+      SET_REF_VALUE (1, iPos);
+      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sFilter, &sDqLayer, iEdge, 1,
                    0) == uiBSx4) << iEdge << " " << iPos << " diff_mv == 4";
 
       UT_DB_CLEAN_STATUS
       iLayerMv[0][0 * 16 + iCurrBlock][0] = -2048;
       iLayerMv[0][1 * 16 + iNeighborBlock][0] = 2047;
-      SET_REF_VALUE(1, iPos);
-      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sDqLayer, iEdge, 1,
+      SET_REF_VALUE (1, iPos);
+      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sFilter, &sDqLayer, iEdge, 1,
                    0) == uiBSx4) << iEdge << " " << iPos << " diff_mv == maximum";
 
       UT_DB_CLEAN_STATUS
       iLayerMv[0][0 * 16 + iCurrBlock][1] = -2048;
       iLayerMv[0][1 * 16 + iNeighborBlock][1] = 2047;
-      SET_REF_VALUE(1, iPos);
-      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sDqLayer, iEdge, 1,
+      SET_REF_VALUE (1, iPos);
+      EXPECT_TRUE (DeblockingBsMarginalMBAvcbase (&sFilter, &sDqLayer, iEdge, 1,
                    0) == uiBSx4) << iEdge << " " << iPos << " diff_mv == maximum";
     }
   }
@@ -889,7 +915,8 @@ TEST (Deblocking, WelsDeblockingMb) {
   // void WelsDeblockingMb (PDqLayer pCurDqLayer, PDeblockingFilter  pFilter, int32_t iBoundryFlag)
   /* Deblock one MB, calculate the Bs inside the function, only consider the intra / intra block */
   SDqLayer sDqLayer;
-
+  sDqLayer.sLayerInfo.sSliceInLayer.sSliceHeaderExt.sSliceHeader.eSliceType = P_SLICE;
+  sDqLayer.pDec = NULL;
   SDeblockingFilter sFilter;
   SDeblockingFunc sDBFunc;
   sFilter.pLoopf = &sDBFunc;
@@ -901,6 +928,7 @@ TEST (Deblocking, WelsDeblockingMb) {
   sFilter.pLoopf->pfLumaDeblockingEQ4Hor = &UT_DeblockingFuncLumaEQ4Func;
   sFilter.pLoopf->pfLumaDeblockingLT4Ver = &UT_DeblockingFuncLumaLT4Func;
   sFilter.pLoopf->pfLumaDeblockingEQ4Ver = &UT_DeblockingFuncLumaEQ4Func;
+  sFilter.pRefPics[0] = sFilter.pRefPics[1] = NULL; // Don't need Ref pics for intra tests
 
   sDqLayer.iMbX = sDqLayer.iMbY = 0;
   sDqLayer.iMbXyIndex = 1;
@@ -925,7 +953,7 @@ TEST (Deblocking, WelsDeblockingMb) {
   sDqLayer.pLumaQp = iLumaQP;
   sDqLayer.pChromaQp = iChromaQP;
 
-  int16_t iMbType[2];
+  uint32_t iMbType[2];
   sDqLayer.pMbType = iMbType;
   sDqLayer.pMbType[0] = MB_TYPE_INTRA4x4;
   sDqLayer.pMbType[1] = MB_TYPE_INTRA4x4;
